# -*- coding: utf-8 -*-
from setuphelpers import *
import platform


r"""
Usable WAPT package functions: install(), uninstall(), session_setup(), audit(), update_package()

"""
# Declaring global variables - Warnings: 1) WAPT context is only available in package functions; 2) Global variables are not persistent between calls
bin_contains = "gvim"
silentflags = "/S"


def install():
    # Declaring local variables
    package_version = control.get_software_version()
    bin_name = glob.glob("*%s*.exe" % bin_contains)[0]

    # Uninstalling older versions of the software
    for to_uninstall in installed_softwares(name=r"Vim "):
        if Version(to_uninstall["version"]) < Version(package_version) or force:
            print("Removing: %s (%s)" % (to_uninstall["name"], to_uninstall["version"]))
            killalltasks(control.impacted_process.split(","))
            run(uninstall_cmd(to_uninstall["key"]))
            wait_uninstallkey_absent(to_uninstall["key"])
            package_version = control.get_software_version()
            # removing old version from system path
            path_dir = makepath(programfiles32, "Vim", "vim%s" % to_uninstall["version"].replace(".", ""))
            remove_from_system_path(path_dir)

    # Installing the software
    print("Installing: %s" % bin_name)

    install_exe_if_needed(
        bin_name,
        silentflags=silentflags,
        name="Vim %s" % package_version.rsplit(".", 1)[0],
        min_version=package_version,
    )
    # adding to system path
    destdir = makepath(programfiles32, "Vim", "vim%s" % package_version.rsplit(".", 1)[0].replace(".", ""))
    print("adding to system path : %s " % destdir)
    add_to_system_path(destdir)


def uninstall():
    # Declaring local variables
    package_version = control.get_software_version()
    # removing from system path
    destdir = makepath(programfiles32, "Vim", "vim%s" % package_version.rsplit(".", 1)[0].replace(".", ""))
    remove_from_system_path(destdir)


def update_package():
    # Declaring local variables
    result = False
    proxies = get_proxies()
    if not proxies:
        proxies = get_proxies_from_wapt_console()
    app_name = control.name

    # Getting latest version information from official sources
    url = "https://www.vim.org/download.php"
    dl_url = wgets(url, proxies=proxies).split("The current stable version is")[1].split('href="')[1].split('">')[0]
    if control.architecture == "x64":
        dl_url = dl_url.replace("x86", "x64")
    version = dl_url.split("/")[-2].replace("v", "")
    latest_bin = dl_url.split("/")[-1]

    print("Latest %s version is: %s" % (app_name, version))
    print("Download URL is: %s" % dl_url)

    # Downloading latest binaries
    if not isfile(latest_bin):
        print("Downloading: %s" % latest_bin)
        wget(dl_url, latest_bin, proxies=proxies)

        # Checking version from file
        version_from_file = get_version_from_binary(latest_bin)
        if version_from_file.endswith(".0"):
            version_from_file = version_from_file.rsplit(".", 1)[0]
        # if not version_from_file.startswith(version) and version_from_file != '':
        if Version(version_from_file) != Version(version) and version_from_file != "":
            print("Changing version to the version number of the binary")
            os.rename(latest_bin, bin_contains + version_from_file + ".exe")
            version = version_from_file
        else:
            print("Binary file version corresponds to online version")

    version = ".".join(version.split(".")[:2])
    # Changing version of the package
    if Version(version) > Version(control.get_software_version()):
        print("Software version updated (from: %s to: %s)" % (control.get_software_version(), Version(version)))
        result = True
    else:
        print("Software version up-to-date (%s)" % Version(version))
    control.version = "%s-%s" % (Version(version), control.version.split("-", 1)[-1])
    # control.set_software_version(version)
    control.save_control_to_wapt()

    # Deleting outdated binaries
    remove_outdated_binaries(version)

    # Validating update-package-sources
    return result


def remove_outdated_binaries(version, list_extensions=["exe", "msi", "deb", "rpm", "dmg", "pkg"], filename_contains=None):
    r"""Remove files based on the version contained in his filename, failing over on file version on compatible OSes

    Args:
        version (str): version number of keeped files
        list_extensions (str or list of str): file extensions of compared files
        filename_contains (str or list of str): Part of the filename that must be contained (useful for distinguishing architecture and os)

    Returns:
        list: list of deleted files

    .. versionadded:: 2.0

    .. versionchanged:: 2.2
        Now returns removed files, now checking .exe and .msi file versions

    """
    files = []
    if type(list_extensions) != list:
        list_extensions = [list_extensions]
    if filename_contains:
        if type(filename_contains) != list:
            filename_contains = [filename_contains]
    list_extensions = ["." + ext for ext in list_extensions if ext[0] != "."]
    for file_ext in list_extensions:
        for bin_in_dir in glob.glob("*%s" % file_ext):
            if not version in bin_in_dir:
                if platform.system() == "Windows":
                    if file_ext == ".exe" or file_ext == ".msi":
                        if Version(version) == Version(get_version_from_binary(bin_in_dir, "FileVersion")) or Version(version) == Version(
                            get_version_from_binary(bin_in_dir, "ProductVersion")
                        ):
                            print("%s file or product version is correct (%s)" % (bin_in_dir, version))
                            continue
                remove_file(bin_in_dir)
                files.append(bin_in_dir)
            if filename_contains:
                for filename_contain in filename_contains:
                    if not filename_contain in bin_in_dir:
                        remove_file(bin_in_dir)
                        files.append(bin_in_dir)
    return [fn for fn in files]
